@@ -157,19 +157,27 @@ class MatriceDeployServer:
     def _register_shutdown_handlers(self):
         """Register signal handlers and atexit callback for graceful shutdown."""
         def signal_handler(signum, frame):
-            logging.info("Received signal %d, initiating graceful shutdown...", signum)
+            logging.info("Received signal %d, triggering shutdown through utils...", signum)
             try:
-                self.stop_server()
+                # Use utils shutdown to trigger coordinated shutdown
+                if hasattr(self, 'utils') and self.utils:
+                    self.utils._shutdown_initiated.set()
+                else:
+                    # Fallback to direct shutdown if utils not available
+                    self.stop_server()
+                    os._exit(0)
             except Exception as exc:
                 logging.error("Error during signal-triggered shutdown: %s", str(exc))
-            finally:
-                os._exit(0)
+                os._exit(1)
 
         def atexit_handler():
             logging.info("Process exiting, ensuring graceful shutdown...")
             try:
                 if not self._shutdown_event.is_set():
-                    self.stop_server()
+                    if hasattr(self, 'utils') and self.utils and not self.utils._shutdown_initiated.is_set():
+                        self.utils._shutdown_initiated.set()
+                    else:
+                        self.stop_server()
             except Exception as exc:
                 logging.error("Error during atexit shutdown: %s", str(exc))
 
@@ -214,7 +222,7 @@ class MatriceDeployServer:
                 f"Invalid external port: {external_port}. Must be between 1 and 65535"
             )
 
-    def start(self):
+    def start(self, block=True):
         """Start the proxy interface and all server components."""
         try:
             self._validate_configuration()
@@ -231,7 +239,7 @@ class MatriceDeployServer:
                 "Model deployment model loaded",
             )
             self.utils = MatriceDeployServerUtils(
-                self.action_tracker, self.inference_interface, self.external_port
+                self.action_tracker, self.inference_interface, self.external_port, self
             )
             self.utils.update_deployment_address()
             self.utils.run_background_checkers()
@@ -240,6 +248,8 @@ class MatriceDeployServer:
                 "SUCCESS",
                 "Model deployment started",
             )
+            if block:
+                self.utils.wait_for_shutdown()
         except Exception as exc:
             logging.error("Failed to start server components: %s", str(exc))
             self.action_tracker.update_status(
@@ -394,7 +404,7 @@ class MatriceDeployServer:
             self._stream_manager_thread = threading.Thread(
                 target=run_stream_manager_in_thread,
                 name="StreamManagerThread",
-                daemon=True
+                daemon=False
             )
             self._stream_manager_thread.start()
             logging.info("Stream manager thread started successfully")
@@ -420,13 +430,16 @@ class MatriceDeployServer:
         self.proxy_interface.start()
         logging.info("Proxy interface started successfully")
 
-    def start_server(self):
+    def start_server(self, block=True):
         """Start the server and related components.
 
+        Args:
+            block: If True, wait for shutdown signal. If False, return immediately after starting.
+
         Raises:
             Exception: If unable to initialize server
         """
-        self.start()
+        self.start(block=block)
 
     def stop_server(self):
         """Stop the server and related components."""
@@ -471,6 +484,7 @@ class MatriceDeployServerUtils:
         action_tracker: ActionTracker,
         inference_interface: InferenceInterface,
         external_port: int,
+        main_server: 'MatriceDeployServer' = None,
     ):
         """Initialize utils with reference to the main server.
 
@@ -478,6 +492,7 @@ class MatriceDeployServerUtils:
             action_tracker: ActionTracker instance
             inference_interface: InferenceInterface instance
             external_port: External port number
+            main_server: Reference to the main MatriceDeployServer instance
         """
         self.action_tracker = action_tracker
         self.session = self.action_tracker.session
@@ -492,9 +507,14 @@ class MatriceDeployServerUtils:
         self.auto_shutdown = self.action_details.get("autoShutdown", True)
         self.inference_interface = inference_interface
         self.external_port = external_port
+        self.main_server = main_server
         self._ip = None
         self._ip_fetch_attempts = 0
         self._max_ip_fetch_attempts = MAX_IP_FETCH_ATTEMPTS
+        
+        # Shutdown coordination
+        self._shutdown_initiated = threading.Event()
+        self._shutdown_complete = threading.Event()
 
     @property
     def ip(self):
@@ -658,9 +678,6 @@ class MatriceDeployServerUtils:
                     "Exception while notifying backend of shutdown: %s", str(exc)
                 )
 
-            # Allow time for cleanup
-            time.sleep(CLEANUP_DELAY_SECONDS)
-
             # Update status
             try:
                 self.action_tracker.update_status(
@@ -672,13 +689,23 @@ class MatriceDeployServerUtils:
             except Exception as exc:
                 logging.error("Failed to update deployment status: %s", str(exc))
 
-            logging.info("Waiting for final cleanup...")
-            time.sleep(FINAL_CLEANUP_DELAY_SECONDS)
-            logging.info("Shutdown complete, exiting process")
+            # Signal shutdown initiation instead of direct exit
+            logging.info("Signaling shutdown to main thread...")
+            self._shutdown_initiated.set()
+            
+            # Wait for coordinated shutdown to complete or timeout
+            if self._shutdown_complete.wait(timeout=30.0):
+                logging.info("Coordinated shutdown completed, exiting process")
+            else:
+                logging.warning("Coordinated shutdown timed out, forcing exit")
+            
+            # Final exit
             os._exit(0)
 
         except Exception as exc:
             logging.error("Error during shutdown: %s", str(exc))
+            # Signal shutdown even on error
+            self._shutdown_initiated.set()
             os._exit(1)
 
     def shutdown_checker(self):
@@ -836,20 +863,62 @@ class MatriceDeployServerUtils:
         shutdown_thread = threading.Thread(
             target=self.shutdown_checker,
             name="ShutdownChecker",
+            daemon=False,
         )
         heartbeat_thread = threading.Thread(
             target=self.heartbeat_checker,
             name="HeartbeatChecker",
+            daemon=False,
         )
 
-        shutdown_thread.daemon = True
-        heartbeat_thread.daemon = True
-
         shutdown_thread.start()
         heartbeat_thread.start()
 
         logging.info("Background checker threads started successfully")
 
+    def wait_for_shutdown(self):
+        """Wait for shutdown to be initiated by background checkers or external signals.
+        
+        This method blocks the main thread until shutdown is triggered.
+        """
+        try:
+            logging.info("Main thread waiting for shutdown signal...")
+            
+            # Wait for shutdown to be initiated
+            self._shutdown_initiated.wait()
+            
+            logging.info("Shutdown signal received, initiating server shutdown...")
+            
+            # Trigger coordinated shutdown
+            if self.main_server:
+                try:
+                    self.main_server.stop_server()
+                    logging.info("Server shutdown completed")
+                except Exception as exc:
+                    logging.error("Error during server shutdown: %s", str(exc))
+            
+            # Signal that shutdown is complete
+            self._shutdown_complete.set()
+            
+        except KeyboardInterrupt:
+            logging.info("Received KeyboardInterrupt, initiating shutdown...")
+            self._shutdown_initiated.set()
+            if self.main_server:
+                try:
+                    self.main_server.stop_server()
+                except Exception as exc:
+                    logging.error("Error during keyboard interrupt shutdown: %s", str(exc))
+            self._shutdown_complete.set()
+        except Exception as exc:
+            logging.error("Error in wait_for_shutdown: %s", str(exc))
+            self._shutdown_initiated.set()
+            if self.main_server:
+                try:
+                    self.main_server.stop_server()
+                except Exception as exc:
+                    logging.error("Error during exception shutdown: %s", str(exc))
+            self._shutdown_complete.set()
+
     def update_deployment_address(self):
         """Update the deployment address in the backend.
 
