@@ -18,24 +18,37 @@ from ..utils import (
     BBoxSmoothingConfig,
     BBoxSmoothingTracker
 )
-try:
-    import cv2
-    import numpy as np
-    import torch
-    from ..ocr.easyocr_extractor import EasyOCRExtractor
-    from ..ocr.postprocessing import TextPostprocessor 
-    from ..ocr.preprocessing import ImagePreprocessor
-    from ..core.config import BaseConfig, AlertConfig, ZoneConfig
-    
-except:
-    print("test-------------------------------------------------")
+# External dependencies
+import cv2
+import numpy as np
+#import torch
+import re
 
+# Fast license-plate OCR (replaces EasyOCR)
+# Attempt to import fast_plate_ocr; fall back to a stub if unavailable
+try:
+    from fast_plate_ocr import LicensePlateRecognizer  # type: ignore
+except ImportError:  # pragma: no cover – optional dependency may be absent
+    class LicensePlateRecognizer:  # type: ignore
+        """Stub fallback when fast_plate_ocr is not installed."""
+        def __init__(self, *args, **kwargs):
+            raise ImportError("fast_plate_ocr is required for LicensePlateMonitorUseCase but is not installed.")
+
+# Internal utilities that are still required
+from ..ocr.preprocessing import ImagePreprocessor
+from ..core.config import BaseConfig, AlertConfig, ZoneConfig
+
+# (Catch import errors early in the logs)
+try:
+    _ = LicensePlateRecognizer  # noqa: B018 – reference to quiet linters
+except Exception as _e:
+    print(f"Warning: fast_plate_ocr could not be imported ⇒ {_e}")
 
 
 @dataclass
 class LicensePlateMonitorConfig(BaseConfig):
     """Configuration for License plate detection use case in License plate monitoring."""
-    enable_smoothing: bool = True
+    enable_smoothing: bool = False
     smoothing_algorithm: str = "observability"  # "window" or "observability"
     smoothing_window_size: int = 20
     smoothing_cooldown_frames: int = 5
@@ -44,12 +57,12 @@ class LicensePlateMonitorConfig(BaseConfig):
     frame_skip: int = 1
     fps: Optional[float] = None
     bbox_format: str = "auto"
-    usecase_categories: List[str] = field(default_factory=lambda: ['License_Plate'])
-    target_categories: List[str] = field(default_factory=lambda: ['License_Plate'])
+    usecase_categories: List[str] = field(default_factory=lambda: ['license_plate'])
+    target_categories: List[str] = field(default_factory=lambda: ['license_plate'])
     alert_config: Optional[AlertConfig] = None
-    index_to_category: Optional[Dict[int, str]] = field(default_factory=lambda: {0: "License_Plate"})
-    language: List[str] = field(default_factory=lambda: ['en', 'ar'])
-    country: str = field(default_factory=lambda: 'qatar')
+    index_to_category: Optional[Dict[int, str]] = field(default_factory=lambda: {0: "license_plate"})
+    language: List[str] = field(default_factory=lambda: ['en'])
+    country: str = field(default_factory=lambda: 'us')
 
     def validate(self) -> List[str]:
         """Validate configuration parameters."""
@@ -69,12 +82,19 @@ class LicensePlateMonitorConfig(BaseConfig):
         return errors
 
 class LicensePlateMonitorUseCase(BaseProcessor):
-    CATEGORY_DISPLAY = {"License_Plate": "License Plate"}
+    CATEGORY_DISPLAY = {"license_plate": "license_plate"}
+    
+    # --------------------------------------------------------------
+    # Shared resources (initialised once per process)
+    # --------------------------------------------------------------
+    _ocr_model: Optional[LicensePlateRecognizer] = None  # Fast plate OCR
+    _non_alnum_regex = re.compile(r"[^A-Za-z0-9]+")
+
     
     def __init__(self):
         super().__init__("license_plate_monitor")
         self.category = "license_plate_monitor"
-        self.target_categories = ['License_Plate']
+        self.target_categories = ['license_plate']
         self.CASE_TYPE: Optional[str] = 'license_plate_monitor'
         self.CASE_VERSION: Optional[str] = '1.3'
         self.smoothing_tracker = None
@@ -91,13 +111,28 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         self._seen_plate_texts = set()
         # CHANGE: Added _tracked_plate_texts to store the longest plate_text per track_id
         self._tracked_plate_texts: Dict[Any, str] = {}
+        # Containers for text stability & uniqueness
+        self._unique_plate_texts: Dict[str, str] = {}  # cleaned_text -> original (longest)
         self.image_preprocessor = ImagePreprocessor()
-        self.ocr_extractor = None  # Initialized in process
-        self.text_postprocessor = TextPostprocessor()
+        # Fast OCR model (shared across instances)
+        if LicensePlateMonitorUseCase._ocr_model is None:
+            try:
+                LicensePlateMonitorUseCase._ocr_model = LicensePlateRecognizer('cct-s-v1-global-model')
+                self.logger.info("LicensePlateRecognizer loaded successfully")
+            except Exception as e:
+                self.logger.error(f"Failed to initialise LicensePlateRecognizer: {e}")
+        self.ocr_model = LicensePlateMonitorUseCase._ocr_model
+        # OCR text history for stability checks (text → consecutive frame count)
+        self._text_history: Dict[str, int] = {}
 
         self.start_timer = None
         #self.reset_timer = "2025-08-19-04:22:47.187574 UTC"
 
+        # Minimum length for a valid plate (after cleaning)
+        self._min_plate_len = 5
+        # number of consecutive frames a plate must appear to be considered "stable"
+        self._stable_frames_required = 3
+
     def reset_tracker(self) -> None:
         """Reset the advanced tracker instance."""
         if self.tracker is not None:
@@ -111,6 +146,8 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         self._tracked_plate_texts = {}
         self._total_frame_counter = 0
         self._global_frame_offset = 0
+        self._text_history = {}
+        self._unique_plate_texts = {}
         self.logger.info("Plate tracking state reset")
 
     def reset_all_tracking(self) -> None:
@@ -121,8 +158,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
 
     def process(self, data: Any, config: ConfigProtocol, input_bytes: Optional[bytes] = None, 
                 context: Optional[ProcessingContext] = None, stream_info: Optional[Dict[str, Any]] = None) -> ProcessingResult:
-        start_time = time.time()
-        
+        processing_start = time.time()
         try:
             if not isinstance(config, LicensePlateMonitorConfig):
                 return self.create_error_result("Invalid configuration type for license plate monitoring",
@@ -135,19 +171,19 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 return self.create_error_result("input_bytes (video/image) is required for license plate monitoring",
                                                usecase=self.name, category=self.category, context=context)
             
-            # if not data:
-            #     return self.create_result(
-            #     data={"agg_summary": {}},
-            #     usecase=self.name,
-            #     category=self.category,
-            #     context=context
-            # )
-                # return self.create_error_result("Detection data is required for license plate monitoring",
-                #                                usecase=self.name, category=self.category, context=context)
-            
             # Initialize OCR extractor if not already done
-            if self.ocr_extractor is None:
-                self.ocr_extractor = EasyOCRExtractor(lang=config.language, gpu=torch.cuda.is_available(), verbose=False)
+            if self.ocr_model is None:
+                self.logger.info("Lazy initialisation fallback (should rarely happen)")
+                try:
+                    LicensePlateMonitorUseCase._ocr_model = LicensePlateRecognizer('cct-s-v1-global-model')
+                    self.ocr_model = LicensePlateMonitorUseCase._ocr_model
+                except Exception as e:
+                    return self.create_error_result(
+                        f"Failed to initialise OCR model: {e}",
+                        usecase=self.name,
+                        category=self.category,
+                        context=context,
+                    )
             
             input_format = match_results_structure(data)
             context.input_format = input_format
@@ -155,21 +191,23 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             
             self.logger.info(f"Processing license plate monitoring with format: {input_format.value}")
             
-            # Step 1: Apply confidence filtering
+            # Step 1: Apply confidence filtering 1
+            # print("---------CONFIDENCE FILTERING",config.confidence_threshold)
+            # print("---------DATA1--------------",data)
             processed_data = filter_by_confidence(data, config.confidence_threshold)
-            self.logger.debug(f"Applied confidence filtering with threshold {config.confidence_threshold}")
+            #self.logger.debug(f"Applied confidence filtering with threshold {config.confidence_threshold}")
             
             # Step 2: Apply category mapping if provided
             if config.index_to_category:
                 processed_data = apply_category_mapping(processed_data, config.index_to_category)
-                self.logger.debug("Applied category mapping")
-            
+                #self.logger.debug("Applied category mapping")
+            #print("---------DATA2--------------",processed_data)
             # Step 3: Filter to target categories
             processed_data = [d for d in processed_data if d.get('category') in self.target_categories]
-            self.logger.debug("Applied category filtering")
+            #self.logger.debug("Applied category filtering")
             
             raw_processed_data = [copy.deepcopy(det) for det in processed_data]
-            
+            #print("---------DATA2--------------",processed_data)
             # Step 4: Apply bounding box smoothing if enabled
             if config.enable_smoothing:
                 if self.smoothing_tracker is None:
@@ -190,9 +228,9 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 from ..advanced_tracker.config import TrackerConfig
                 if self.tracker is None:
                     tracker_config = TrackerConfig(
-                        track_high_thresh=float(config.confidence_threshold),
+                        track_high_thresh=float(config.confidence_threshold-0.1),
                         track_low_thresh=max(0.05, float(config.confidence_threshold) / 2),
-                        new_track_thresh=float(config.confidence_threshold)
+                        new_track_thresh=float(config.confidence_threshold-0.1)
                     )
                     self.tracker = AdvancedTracker(tracker_config)
                     self.logger.info(f"Initialized AdvancedTracker with thresholds: high={tracker_config.track_high_thresh}, "
@@ -200,17 +238,20 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 processed_data = self.tracker.update(processed_data)
             except Exception as e:
                 self.logger.warning(f"AdvancedTracker failed: {e}")
-            
+            #print("---------DATA3--------------",processed_data)
             # Step 6: Update tracking state
             self._update_tracking_state(processed_data)
-            
+            #print("---------DATA4--------------",processed_data)
             # Step 7: Attach masks to detections
             processed_data = self._attach_masks_to_detections(processed_data, raw_processed_data)
-            
+            #print("---------DATA5--------------",processed_data)
             # Step 8: Perform OCR on media
             ocr_analysis = self._analyze_ocr_in_media(processed_data, input_bytes, config)
+
+            #print("ocr_analysis", ocr_analysis)
             
             # Step 9: Update plate texts
+            #print("---------DATA6--------------",processed_data)
             processed_data = self._update_detections_with_ocr(processed_data, ocr_analysis)
             self._update_plate_texts(processed_data)
             
@@ -257,6 +298,16 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 category=self.category,
                 context=context
             )
+            proc_time = time.time() - processing_start
+            processing_latency_ms = proc_time * 1000.0
+            processing_fps = (1.0 / proc_time) if proc_time > 0 else None
+            # Log the performance metrics using the module-level logger
+            print(
+                "Processing completed in ms, fps for fps:",
+                processing_latency_ms,
+                processing_fps,
+                self._total_frame_counter
+            )
             return result
             
         except Exception as e:
@@ -281,84 +332,8 @@ class LicensePlateMonitorUseCase(BaseProcessor):
 
     def _analyze_ocr_in_media(self, data: Any, media_bytes: bytes, config: LicensePlateMonitorConfig) -> List[Dict[str, Any]]:
         """Analyze OCR of license plates in video frames or images."""
-        is_video = self._is_video_bytes(media_bytes)
-        if is_video:
-            return self._analyze_ocr_in_video(data, media_bytes, config)
-        else:
-            return self._analyze_ocr_in_image(data, media_bytes, config)
+        return self._analyze_ocr_in_image(data, media_bytes, config)
 
-    def _analyze_ocr_in_video(self, data: Any, video_bytes: bytes, config: LicensePlateMonitorConfig) -> List[Dict[str, Any]]:
-        """Analyze OCR in video frames."""
-        with tempfile.NamedTemporaryFile(suffix=".mp4", delete=False) as temp_video:
-            temp_video.write(video_bytes)
-            video_path = temp_video.name
-
-        try:
-            cap = cv2.VideoCapture(video_path)
-            if not cap.isOpened():
-                raise RuntimeError("Failed to open video file")
-
-            fps = config.fps or cap.get(cv2.CAP_PROP_FPS)
-            ocr_analysis = []
-            frame_id = 0
-
-            while True:
-                ret, frame = cap.read()
-                if not ret:
-                    break
-
-                if frame_id % config.frame_skip != 0:
-                    frame_id += 1
-                    continue
-
-                frame_key = str(frame_id)
-                timestamp = frame_id / fps
-                frame_detections = self._get_frame_detections(data, frame_key)
-                if not frame_detections:
-                    frame_id += 1
-                    continue
-
-                rgb_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
-                for detection in frame_detections:
-                    if detection.get("confidence", 1.0) < config.confidence_threshold:
-                        continue
-
-                    bbox = detection.get("bounding_box", detection.get("bbox"))
-                    if not bbox:
-                        continue
-
-                    crop = self._crop_bbox(rgb_frame, bbox, config.bbox_format)
-                    if crop.size == 0:
-                        continue
-
-                    preprocessed_crop = self.image_preprocessor.preprocess(crop, grayscale=True)
-                    ocr_results = self.ocr_extractor.extract(preprocessed_crop, detail=1, paragraph=False)
-                    texts = [r['text'] for r in ocr_results]
-                    confs = [r['confidence'] for r in ocr_results]
-                    processed = self.text_postprocessor.postprocess(texts, confs, task="license_plate", region=config.country, confidence_threshold=0.3)
-                    valid_processed = [p for p in processed if p[2]]
-                    plate_text = valid_processed[0][0] if valid_processed else None
-
-                    ocr_record = {
-                        "frame_id": frame_key,
-                        "timestamp": round(timestamp, 2),
-                        "category": detection.get("category", "License_Plate"),
-                        "confidence": round(detection.get("confidence", 0.0), 3),
-                        "plate_text": plate_text,
-                        "bbox": bbox,
-                        "detection_id": detection.get("id", f"det_{len(ocr_analysis)}"),
-                        "track_id": detection.get("track_id")
-                    }
-                    ocr_analysis.append(ocr_record)
-
-                frame_id += 1
-
-            cap.release()
-            return ocr_analysis
-
-        finally:
-            if os.path.exists(video_path):
-                os.unlink(video_path)
 
     def _analyze_ocr_in_image(self, data: Any, image_bytes: bytes, config: LicensePlateMonitorConfig) -> List[Dict[str, Any]]:
         """Analyze OCR in a single image."""
@@ -371,26 +346,27 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         rgb_image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
         ocr_analysis = []
         detections = self._get_frame_detections(data, "0")
+
+        #print("OCR-detections", detections)
         
         for detection in detections:
+            #print("---------OCR DETECTION",detection)
             if detection.get("confidence", 1.0) < config.confidence_threshold:
                 continue
 
             bbox = detection.get("bounding_box", detection.get("bbox"))
+            #print("---------OCR BBOX",bbox)
             if not bbox:
                 continue
 
             crop = self._crop_bbox(rgb_image, bbox, config.bbox_format)
+            #print("---------OCR CROP SIZEE",crop.size)
             if crop.size == 0:
                 continue
-
-            preprocessed_crop = self.image_preprocessor.preprocess(crop, grayscale=True)
-            ocr_results = self.ocr_extractor.extract(preprocessed_crop, detail=1, paragraph=False)
-            texts = [r['text'] for r in ocr_results]
-            confs = [r['confidence'] for r in ocr_results]
-            processed = self.text_postprocessor.postprocess(texts, confs, task="license_plate", region=config.country, confidence_threshold=0.3)
-            valid_processed = [p for p in processed if p[2]]
-            plate_text = valid_processed[0][0] if valid_processed else None
+            
+            plate_text_raw = self._run_ocr(crop)
+            #print("---------OCR PLATE TEXT",plate_text_raw)
+            plate_text = plate_text_raw if plate_text_raw else None
 
             ocr_record = {
                 "frame_id": "0",
@@ -433,6 +409,34 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             
         return image[ymin:ymax, xmin:xmax]
 
+    # ------------------------------------------------------------------
+    # Fast OCR helpers
+    # ------------------------------------------------------------------
+    def _clean_text(self, text: str) -> str:
+        """Sanitise OCR output to keep only alphanumerics and uppercase."""
+        if not text:
+            return ""
+        return LicensePlateMonitorUseCase._non_alnum_regex.sub('', text).upper()
+
+    def _run_ocr(self, crop: np.ndarray) -> str:
+        """Run OCR on a cropped plate image and return cleaned text or empty string."""
+        # print("---------OCR CROP22",crop)
+        # print("---------OCR CROP SIZE22",crop.size)
+
+        if crop is None or crop.size == 0 or self.ocr_model is None:
+            return ""
+        try:
+            # fast_plate_ocr expects RGB
+            #rgb_crop = cv2.cvtColor(crop, cv2.COLOR_BGR2RGB)
+            res = self.ocr_model.run(crop)
+            if isinstance(res, list):
+                res = res[0] if res else ""
+            cleaned = self._clean_text(str(res))
+            return cleaned if len(cleaned) >= self._min_plate_len else ""
+        except Exception as exc:
+            self.logger.warning(f"OCR failed: {exc}")
+            return ""
+
     def _get_frame_detections(self, data: Any, frame_key: str) -> List[Dict[str, Any]]:
         """Extract detections for a specific frame from data."""
         if isinstance(data, dict):
@@ -456,7 +460,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                     bbox_key = tuple(sorted(rec["bbox"].items())) if rec.get("bbox") else None
                     if bbox_key:
                         ocr_dict[bbox_key] = rec["plate_text"]
-                self.logger.debug(f"OCR record: track_id={track_id}, plate_text={rec.get('plate_text')}, bbox={rec.get('bbox')}")
+                self.logger.info(f"OCR record: track_id={track_id}, plate_text={rec.get('plate_text')}, bbox={rec.get('bbox')}")
         
         for det in detections:
             track_id = det.get("track_id")
@@ -467,31 +471,46 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             elif bbox_key and bbox_key in ocr_dict:
                 plate_text = ocr_dict[bbox_key]
             det["plate_text"] = plate_text
-            self.logger.debug(f"Detection track_id={track_id}, bbox={det.get('bounding_box')}: Assigned plate_text={plate_text}")
+            self.logger.info(f"Detection track_id={track_id}, bbox={det.get('bounding_box')}: Assigned plate_text={plate_text}")
         return detections
 
     def _count_categories(self, detections: List[Dict], config: LicensePlateMonitorConfig) -> Dict[str, Any]:
-        """Count detections per category and include plate texts."""
-        counts = {}
-        valid_detections = []
+        """Count unique licence-plate texts per frame and attach detections."""
+        unique_texts: set = set()
+        valid_detections: List[Dict[str, Any]] = []
+
+        #print("---------SUMMARY COUNT CATEGORIES",detections)
         for det in detections:
             if not all(k in det for k in ['category', 'confidence', 'bounding_box']):
-                self.logger.warning(f"Skipping invalid detection: {det}")
                 continue
-            cat = det.get('category', 'License_Plate')
-            counts[cat] = counts.get(cat, 0) + 1
+
+            cat = det.get('category', 'license_plate')
+            plate_text_raw = det.get('plate_text', '')
+            #print("---------SUMMARY COUNT CATEGORIES PLATE TEXT RAW",plate_text_raw)
+            #print("---------SUMMARY COUNT CATEGORIES PLATE TEXT RAW",det)
+            cleaned_text = self._clean_text(plate_text_raw)
+
+            # Consider as unique only if meets criteria
+            if cleaned_text and len(cleaned_text) >= self._min_plate_len:
+                unique_texts.add(cleaned_text)
+
             valid_detections.append({
                 "bounding_box": det.get("bounding_box"),
                 "category": cat,
                 "confidence": det.get("confidence"),
-                "track_id": det.get("track_id"),
+                "track_id": det.get('track_id'),
                 "frame_id": det.get("frame_id"),
                 "masks": det.get("masks", []),
-                "plate_text": det.get("plate_text")
+                "plate_text": plate_text_raw
             })
-        self.logger.debug(f"Valid detections after filtering: {len(valid_detections)}")
+
+        counts = {"License_Plate": len(unique_texts)} if unique_texts else {}
+
+        # print("---------SUMMARY COUNT CATEGORIES VALID DETECTIONS",valid_detections)
+        # print(len(unique_texts),'per_cat_count',counts)
+
         return {
-            "total_count": sum(counts.values()),
+            "total_count": len(unique_texts),
             "per_category_count": counts,
             "detections": valid_detections
         }
@@ -502,7 +521,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         tracking_stats = []
         total_detections = counting_summary.get("total_count", 0)
         total_counts = counting_summary.get("total_counts", {})
-        cumulative_total = sum(total_counts.values()) if total_counts else 0
+        cumulative_total = sum(set(total_counts.values())) if total_counts else 0
         per_category_count = counting_summary.get("per_category_count", {})
         track_ids_info = self._get_track_ids_info(counting_summary.get("detections", []))
         current_timestamp = self._get_current_timestamp_str(stream_info, precision=False)
@@ -512,42 +531,65 @@ class LicensePlateMonitorUseCase(BaseProcessor):
         camera_info = self.get_camera_info_from_stream(stream_info)
         
         human_text_lines = []
+        #print("counting_summary", counting_summary)
         human_text_lines.append(f"CURRENT FRAME @ {current_timestamp}:")
         if total_detections > 0:
             category_counts = [f"{count} {cat}" for cat, count in per_category_count.items()]
             detection_text = category_counts[0] + " detected" if len(category_counts) == 1 else f"{', '.join(category_counts[:-1])}, and {category_counts[-1]} detected"
             human_text_lines.append(f"\t- {detection_text}")
-            plate_texts = [det.get("plate_text") for det in counting_summary.get("detections", []) if det.get("plate_text")]
-            if plate_texts:
-                human_text_lines.append(f"\t- License Plates: {', '.join(plate_texts)}")
+            # Deduplicate plate texts for the current frame while preserving order
+            seen_cleaned = set()
+            display_texts = []
+            for det in counting_summary.get("detections", []):
+                raw = det.get("plate_text", "")
+                cleaned = self._clean_text(raw)
+                if cleaned and cleaned not in seen_cleaned:
+                    seen_cleaned.add(cleaned)
+                    # Display the longest raw variant seen in this frame for that cleaned text
+                    longest_variant = max(
+                        [d.get("plate_text", "") for d in counting_summary.get("detections", []) if self._clean_text(d.get("plate_text", "")) == cleaned],
+                        key=len,
+                    )
+                    display_texts.append(longest_variant)
+            if display_texts:
+                human_text_lines.append(f"\t- License Plates: {', '.join(display_texts)}")
         else:
             human_text_lines.append(f"\t- No detections")
         
         human_text_lines.append("")
         human_text_lines.append(f"TOTAL SINCE {start_timestamp}:")
         human_text_lines.append(f"\t- Total Detected: {cumulative_total}")
-        # if total_counts:
-        #     for cat, count in total_counts.items():
-        #         if count > 0:
-        #             human_text_lines.append(f"\t- {cat}: {count}")
-        # CHANGE: Use _tracked_plate_texts instead of _seen_plate_texts for TOTAL SINCE
-        
-        if self._tracked_plate_texts:
+
+        if self._unique_plate_texts:
             human_text_lines.append("\t- Unique License Plates:")
-            for text in sorted(self._tracked_plate_texts.values()):
+            for text in sorted(self._unique_plate_texts.values()):
                 human_text_lines.append(f"\t\t- {text}")
-        
+
         current_counts = [{"category": cat, "count": count} for cat, count in per_category_count.items() if count > 0 or total_detections > 0]
         total_counts_list = [{"category": cat, "count": count} for cat, count in total_counts.items() if count > 0 or cumulative_total > 0]
         
         human_text = "\n".join(human_text_lines)
         detections = []
         for detection in counting_summary.get("detections", []):
+            raw = detection.get("plate_text", "")
+            cleaned = self._clean_text(raw)
+            # if cleaned and cleaned not in seen_cleaned2:
+            #     seen_cleaned2.add(cleaned)
+            #     # Display the longest raw variant seen in this frame for that cleaned text
+            if cleaned:
+                #print("---------CLEANED-------------",cleaned)
+                plate_text = max(
+                    [d.get("plate_text", "") for d in counting_summary.get("detections", []) if self._clean_text(d.get("plate_text", "")) == cleaned],
+                    key=len,
+                )
+            else:
+                plate_text = "license_plate"
+            
             bbox = detection.get("bounding_box", {})
-            category = detection.get("category", "License_Plate")
-            plate_text = detection.get("plate_text", "")
+            category = detection.get("category", "license_plate")
+            #plate_text = detection.get("plate_text", "")
             segmentation = detection.get("masks", detection.get("segmentation", detection.get("mask", [])))
-            detection_obj = self.create_detection_object(category, bbox, segmentation=segmentation, plate_text=plate_text)
+            detection_obj = self.create_detection_object(category, bbox, segmentation=None, plate_text=plate_text)
             detections.append(detection_obj)
         
         alert_settings = []
@@ -562,7 +604,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             })
         
         if alerts:
-            human_text_lines.append(f"Alerts: {alerts[0].get('settings', {})} sent @ {current_timestamp}")
+            human_text_lines.append(f"Alerts: {alerts[0].get('settings', {})}")
         else:
             human_text_lines.append("Alerts: None")
         
@@ -719,19 +761,23 @@ class LicensePlateMonitorUseCase(BaseProcessor):
 
     def _generate_summary(self, summary: Dict, incidents: List, tracking_stats: List, business_analytics: List, alerts: List) -> List[Dict]:
         """Generate a human-readable summary."""
-        lines = {
-            "Application Name": self.CASE_TYPE,
-            "Application Version": self.CASE_VERSION
-        }
-        if incidents:
-            lines["Incidents"] = f"\n\t{incidents[0].get('human_text', 'No incidents detected')}\n"
-        if tracking_stats:
-            lines["Tracking Statistics"] = f"\t{tracking_stats[0].get('human_text', 'No tracking statistics detected')}\n"
-        if business_analytics:
-            lines["Business Analytics"] = f"\t{business_analytics[0].get('human_text', 'No business analytics detected')}\n"
-        if not incidents and not tracking_stats and not business_analytics:
-            lines["Summary"] = "No Summary Data"
-        return [lines]
+        """
+        Generate a human_text string for the tracking_stat, incident, business analytics and alerts.
+        """
+        lines = []
+        lines.append("Application Name: "+self.CASE_TYPE)
+        lines.append("Application Version: "+self.CASE_VERSION)
+        if len(incidents) > 0:
+            lines.append("Incidents: "+f"\n\t{incidents[0].get('human_text', 'No incidents detected')}")
+        if len(tracking_stats) > 0:
+            lines.append("Tracking Statistics: "+f"\t{tracking_stats[0].get('human_text', 'No tracking statistics detected')}")
+        if len(business_analytics) > 0:
+            lines.append("Business Analytics: "+f"\t{business_analytics[0].get('human_text', 'No business analytics detected')}")
+
+        if len(incidents) == 0 and len(tracking_stats) == 0 and len(business_analytics) == 0:
+            lines.append("Summary: "+"No Summary Data")
+
+        return ["\n".join(lines)]
 
     def _update_tracking_state(self, detections: List[Dict]):
         """Track unique track_ids per category."""
@@ -756,19 +802,37 @@ class LicensePlateMonitorUseCase(BaseProcessor):
             text = det.get('plate_text')
             track_id = det.get('track_id')
             if text and track_id is not None:
-                # CHANGE: Update _tracked_plate_texts with the longest text for this track_id
-                current_text = self._tracked_plate_texts.get(track_id, '')
-                if len(text) > len(current_text):
-                    self._tracked_plate_texts[track_id] = text
-                    self.logger.debug(f"Updated track_id={track_id} with new longest plate_text={text}")
-                # Maintain _seen_plate_texts for compatibility
-                self._seen_plate_texts.add(text)
-                self.logger.debug(f"Added plate_text={text} for track_id={track_id} to seen texts")
+                # Clean text again to guarantee consistency
+                text = self._clean_text(text)
+                if len(text) < self._min_plate_len:
+                    continue  # Too short to be a valid plate
+
+                # ----------------------------------------------------------
+                # Update consecutive frame appearance counter
+                # ----------------------------------------------------------
+                self._text_history[text] = self._text_history.get(text, 0) + 1
+
+                # Accept plate as stable only after N consecutive frames
+                if self._text_history[text] >= self._stable_frames_required:
+                    # Track longest string variant per track_id
+                    current_text = self._tracked_plate_texts.get(track_id, '')
+                    if len(text) > len(current_text):
+                        self._tracked_plate_texts[track_id] = text
+                    # Maintain global unique mapping (cleaned -> original longest)
+                    longest_variant = self._unique_plate_texts.get(text, '')
+                    if len(text) > len(longest_variant):
+                        self._unique_plate_texts[text] = text
+                   # self.logger.debug(f"Stable plate confirmed: {text} (track_id={track_id})")
+
+        # Reset counters for texts not seen in this frame
+        current_frame_texts = {self._clean_text(det.get('plate_text', '')) for det in detections if det.get('plate_text')}
+        for t in list(self._text_history.keys()):
+            if t not in current_frame_texts:
+                self._text_history[t] = 0
 
     def get_total_counts(self):
-        """Return total unique track_id count for each category."""
-        # CHANGE: Count unique track_ids in _tracked_plate_texts instead of _seen_plate_texts
-        return {'License_Plate': len(self._tracked_plate_texts)}
+        """Return total unique license plate texts encountered so far."""
+        return {'License_Plate': len(self._unique_plate_texts)}
 
     def _get_track_ids_info(self, detections: List[Dict]) -> Dict[str, Any]:
         """Get detailed information about track IDs."""
@@ -915,8 +979,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
 
     def _get_current_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False, frame_id: Optional[str]=None) -> str:
         """Get formatted current timestamp based on stream type."""
-        print('STREAM INFO-------------------------------')
-        print(stream_info)
+        
         if not stream_info:
             return "00:00:00.00"
         if precision:
@@ -926,8 +989,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 else:
                     start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
                 stream_time_str = self._format_timestamp_for_video(start_time)
-                print('CURRENTTTT TIME-------------------------------',stream_info.get("input_settings", {}).get("stream_time", "NA"))
-
+                
                 return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
             else:
                 return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
@@ -939,7 +1001,7 @@ class LicensePlateMonitorUseCase(BaseProcessor):
                 start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
 
             stream_time_str = self._format_timestamp_for_video(start_time)
-            print('CURRENTTTT TIME-------------------------------',stream_info.get("input_settings", {}).get("stream_time", "NA"))
+           
 
             return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
         else:
