@@ -18,6 +18,7 @@ from ..utils import (
 )
 from dataclasses import dataclass, field
 from ..core.config import BaseConfig, AlertConfig, ZoneConfig
+from ..utils.geometry_utils import get_bbox_center, point_in_polygon, get_bbox_bottom25_center
 
 @dataclass
 class VehicleMonitoringConfig(BaseConfig):
@@ -27,10 +28,10 @@ class VehicleMonitoringConfig(BaseConfig):
     smoothing_window_size: int = 20
     smoothing_cooldown_frames: int = 5
     smoothing_confidence_range_factor: float = 0.5
-    confidence_threshold: float = 0.3
+    confidence_threshold: float = 0.6
+    zone_config: Optional[ZoneConfig] = None
     usecase_categories: List[str] = field(
         default_factory=lambda: [
-            # COCO 2017 80-category label set
             "person", "bicycle", "car", "motorcycle", "airplane", "bus", "train", "truck", "boat",
             "traffic light", "fire hydrant", "stop sign", "parking meter", "bench", "bird", "cat", "dog",
             "horse", "sheep", "cow", "elephant", "bear", "zebra", "giraffe", "backpack", "umbrella",
@@ -45,13 +46,11 @@ class VehicleMonitoringConfig(BaseConfig):
     )
     target_categories: List[str] = field(
         default_factory=lambda: [
-            'car', 'bicycle', 'bus', 'truck', 'motorcycle', 'train', 'boat'
-        ]
+            'car', 'bicycle', 'bus','motorcycle']
     )
     alert_config: Optional[AlertConfig] = None
     index_to_category: Optional[Dict[int, str]] = field(
         default_factory=lambda: {
-                # COCO 2017 80-category index mapping
                 0: "person", 1: "bicycle", 2: "car", 3: "motorcycle", 4: "airplane", 5: "bus",
                 6: "train", 7: "truck", 8: "boat", 9: "traffic light", 10: "fire hydrant",
                 11: "stop sign", 12: "parking meter", 13: "bench", 14: "bird", 15: "cat",
@@ -88,7 +87,7 @@ class VehicleMonitoringUseCase(BaseProcessor):
         self.category = "traffic"
         self.CASE_TYPE: Optional[str] = 'vehicle_monitoring'
         self.CASE_VERSION: Optional[str] = '1.0'
-        self.target_categories = ['car', 'bicycle', 'bus', 'truck', 'motorcycle', 'train', 'boat']
+        self.target_categories = ['car', 'bicycle', 'bus', 'truck', 'motorcycle']
         self.smoothing_tracker = None
         self.tracker = None
         self._total_frame_counter = 0
@@ -100,6 +99,19 @@ class VehicleMonitoringUseCase(BaseProcessor):
         self._track_merge_time_window: float = 7.0
         self._ascending_alert_list: List[int] = []
         self.current_incident_end_timestamp: str = "N/A"
+        self.start_timer = None
+        
+        # Track ID storage for total count calculation
+        self._total_track_ids = set()  # Store all unique track IDs seen across calls
+        self._current_frame_track_ids = set()  # Store track IDs from current frame
+        self._total_count = 0  # Cached total count
+        self._last_update_time = time.time()  # Track when last updated
+        
+        # Zone-based tracking storage
+        self._zone_current_track_ids = {}  # zone_name -> set of current track IDs in zone
+        self._zone_total_track_ids = {}  # zone_name -> set of all track IDs that have been in zone
+        self._zone_current_counts = {}  # zone_name -> current count in zone
+        self._zone_total_counts = {}  # zone_name -> total count that have been in zone
 
     def process(self, data: Any, config: ConfigProtocol, context: Optional[ProcessingContext] = None,
                 stream_info: Optional[Dict[str, Any]] = None) -> ProcessingResult:
@@ -170,14 +182,38 @@ class VehicleMonitoringUseCase(BaseProcessor):
         counting_summary = self._count_categories(processed_data, config)
         total_counts = self.get_total_counts()
         counting_summary['total_counts'] = total_counts
+        counting_summary['categories'] = {}
+        for detection in processed_data:
+            category = detection.get("category", "unknown")
+            counting_summary["categories"][category] = counting_summary["categories"].get(category, 0) + 1
+        
+        zone_analysis = {}
+        if config.zone_config and config.zone_config['zones']:
+            # Convert single frame to format expected by count_objects_in_zones
+            frame_data = processed_data #[frame_detections]
+            zone_analysis = count_objects_in_zones(frame_data, config.zone_config['zones'])
+            
+            # Update zone tracking with current frame data
+            if zone_analysis and config.enable_tracking:
+                enhanced_zone_analysis = self._update_zone_tracking(zone_analysis, processed_data, config)
+                # Merge enhanced zone analysis with original zone analysis
+                for zone_name, enhanced_data in enhanced_zone_analysis.items():
+                    zone_analysis[zone_name] = enhanced_data
+                    
+        print("-----------------------------ZONEEEE-----------------------------------")
+        print(zone_analysis)
+        print("-----------------------------ZONEEEE-----------------------------------")
 
-        alerts = self._check_alerts(counting_summary, frame_number, config)
+        alerts = self._check_alerts(counting_summary,zone_analysis, frame_number, config)
         predictions = self._extract_predictions(processed_data)
 
-        incidents_list = self._generate_incidents(counting_summary, alerts, config, frame_number, stream_info)
-        tracking_stats_list = self._generate_tracking_stats(counting_summary, alerts, config, frame_number, stream_info)
-        business_analytics_list = self._generate_business_analytics(counting_summary, alerts, config, stream_info, is_empty=True)
-        summary_list = self._generate_summary(counting_summary, incidents_list, tracking_stats_list, business_analytics_list, alerts)
+        incidents_list = self._generate_incidents(counting_summary,zone_analysis, alerts, config, frame_number, stream_info)
+        tracking_stats_list = self._generate_tracking_stats(counting_summary,zone_analysis, alerts, config, frame_number, stream_info)
+        print("---------------------------TS--------------------------------------")
+        print(tracking_stats_list)
+        print("---------------------------TS--------------------------------------")
+        business_analytics_list = self._generate_business_analytics(counting_summary,zone_analysis, alerts, config, stream_info, is_empty=True)
+        summary_list = self._generate_summary(counting_summary,zone_analysis, incidents_list, tracking_stats_list, business_analytics_list, alerts)
 
         incidents = incidents_list[0] if incidents_list else {}
         tracking_stats = tracking_stats_list[0] if tracking_stats_list else {}
@@ -188,6 +224,7 @@ class VehicleMonitoringUseCase(BaseProcessor):
             "tracking_stats": tracking_stats,
             "business_analytics": business_analytics,
             "alerts": alerts,
+            "zone_analysis": zone_analysis,
             "human_text": summary}
         }
 
@@ -200,6 +237,84 @@ class VehicleMonitoringUseCase(BaseProcessor):
         )
         return result
 
+    def _update_zone_tracking(self, zone_analysis: Dict[str, Dict[str, int]], detections: List[Dict], config: VehicleMonitoringConfig) -> Dict[str, Dict[str, Any]]:
+        """
+        Update zone tracking with current frame data.
+        
+        Args:
+            zone_analysis: Current zone analysis results
+            detections: List of detections with track IDs
+            
+        Returns:
+            Enhanced zone analysis with tracking information
+        """
+        if not zone_analysis or not config.zone_config or not config.zone_config['zones']:
+            return {}
+        
+        enhanced_zone_analysis = {}
+        zones = config.zone_config['zones']
+        
+        # Get current frame track IDs in each zone
+        current_frame_zone_tracks = {}
+        
+        # Initialize zone tracking for all zones
+        for zone_name in zones.keys():
+            current_frame_zone_tracks[zone_name] = set()
+            if zone_name not in self._zone_current_track_ids:
+                self._zone_current_track_ids[zone_name] = set()
+            if zone_name not in self._zone_total_track_ids:
+                self._zone_total_track_ids[zone_name] = set()
+        
+        # Check each detection against each zone
+        for detection in detections:
+            track_id = detection.get("track_id")
+            if track_id is None:
+                continue
+            
+            # Get detection bbox
+            bbox = detection.get("bounding_box", detection.get("bbox"))
+            if not bbox:
+                continue
+            
+            # Get detection center point
+            center_point = get_bbox_bottom25_center(bbox) #get_bbox_center(bbox)
+            
+            # Check which zone this detection is in using actual zone polygons
+            for zone_name, zone_polygon in zones.items():
+                # Convert polygon points to tuples for point_in_polygon function
+                # zone_polygon format: [[x1, y1], [x2, y2], [x3, y3], ...]
+                polygon_points = [(point[0], point[1]) for point in zone_polygon]
+                
+                # Check if detection center is inside the zone polygon using ray casting algorithm
+                if point_in_polygon(center_point, polygon_points):
+                    current_frame_zone_tracks[zone_name].add(track_id)
+        
+        # Update zone tracking for each zone
+        for zone_name, zone_counts in zone_analysis.items():
+            # Get current frame tracks for this zone
+            current_tracks = current_frame_zone_tracks.get(zone_name, set())
+            
+            # Update current zone tracks
+            self._zone_current_track_ids[zone_name] = current_tracks
+            
+            # Update total zone tracks (accumulate all track IDs that have been in this zone)
+            self._zone_total_track_ids[zone_name].update(current_tracks)
+            
+            # Update counts
+            self._zone_current_counts[zone_name] = len(current_tracks)
+            self._zone_total_counts[zone_name] = len(self._zone_total_track_ids[zone_name])
+            
+            # Create enhanced zone analysis
+            enhanced_zone_analysis[zone_name] = {
+                "current_count": self._zone_current_counts[zone_name],
+                "total_count": self._zone_total_counts[zone_name],
+                "current_track_ids": list(current_tracks),
+                "total_track_ids": list(self._zone_total_track_ids[zone_name]),
+                "original_counts": zone_counts  # Preserve original zone counts
+            }
+        
+        return enhanced_zone_analysis
+
     def _normalize_yolo_results(self, data: Any, index_to_category: Optional[Dict[int, str]] = None) -> Any:
         """
         Normalize YOLO-style outputs to internal detection schema:
@@ -273,7 +388,7 @@ class VehicleMonitoringUseCase(BaseProcessor):
             return normalized_dict
         return data
 
-    def _check_alerts(self, summary: dict, frame_number: Any, config: VehicleMonitoringConfig) -> List[Dict]:
+    def _check_alerts(self, summary: dict, zone_analysis: Dict, frame_number: Any, config: VehicleMonitoringConfig) -> List[Dict]:
         def get_trend(data, lookback=900, threshold=0.6):
             window = data[-lookback:] if len(data) >= lookback else data
             if len(window) < 2:
@@ -306,7 +421,7 @@ class VehicleMonitoringUseCase(BaseProcessor):
                         "threshold_level": threshold,
                         "ascending": get_trend(self._ascending_alert_list, lookback=900, threshold=0.8),
                         "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']),
-                                                         getattr(config.alert_config, 'alert_value', ['JSON']))}
+                                                        getattr(config.alert_config, 'alert_value', ['JSON']))}
                     })
                 elif category in per_category_count and per_category_count[category] > threshold:
                     alerts.append({
@@ -316,12 +431,12 @@ class VehicleMonitoringUseCase(BaseProcessor):
                         "threshold_level": threshold,
                         "ascending": get_trend(self._ascending_alert_list, lookback=900, threshold=0.8),
                         "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']),
-                                                         getattr(config.alert_config, 'alert_value', ['JSON']))}
+                                                        getattr(config.alert_config, 'alert_value', ['JSON']))}
                     })
         return alerts
 
-    def _generate_incidents(self, counting_summary: Dict, alerts: List, config: VehicleMonitoringConfig,
-                           frame_number: Optional[int] = None, stream_info: Optional[Dict[str, Any]] = None) -> List[Dict]:
+    def _generate_incidents(self, counting_summary: Dict, zone_analysis: Dict, alerts: List, config: VehicleMonitoringConfig,
+                        frame_number: Optional[int] = None, stream_info: Optional[Dict[str, Any]] = None) -> List[Dict]:
         incidents = []
         total_detections = counting_summary.get("total_count", 0)
         current_timestamp = self._get_current_timestamp_str(stream_info)
@@ -407,7 +522,7 @@ class VehicleMonitoringUseCase(BaseProcessor):
             incidents.append({})
         return incidents
 
-    def _generate_tracking_stats(self, counting_summary: Dict, alerts: List, config: VehicleMonitoringConfig,
+    def _generate_tracking_stats(self, counting_summary: Dict, zone_analysis: Dict, alerts: List, config: VehicleMonitoringConfig,
                                 frame_number: Optional[int] = None, stream_info: Optional[Dict[str, Any]] = None) -> List[Dict]:
         camera_info = self.get_camera_info_from_stream(stream_info)
         tracking_stats = []
@@ -447,17 +562,49 @@ class VehicleMonitoringUseCase(BaseProcessor):
                 "threshold_level": config.alert_config.count_thresholds if hasattr(config.alert_config, 'count_thresholds') else {},
                 "ascending": True,
                 "settings": {t: v for t, v in zip(getattr(config.alert_config, 'alert_type', ['Default']),
-                                                 getattr(config.alert_config, 'alert_value', ['JSON']))}
+                                                getattr(config.alert_config, 'alert_value', ['JSON']))}
             })
 
         human_text_lines = [f"Tracking Statistics:"]
         human_text_lines.append(f"CURRENT FRAME @ {current_timestamp}")
-        for cat, count in per_category_count.items():
-            human_text_lines.append(f"\t{cat}: {count}")
-        human_text_lines.append(f"TOTAL SINCE {start_timestamp}")
-        for cat, count in total_counts_dict.items():
-            if count > 0:
+        # Append zone-wise current counts if available
+        if zone_analysis:
+            human_text_lines.append("\tZones (current):")
+            for zone_name, zone_data in zone_analysis.items():
+                current_count = 0
+                if isinstance(zone_data, dict):
+                    if "current_count" in zone_data:
+                        current_count = zone_data.get("current_count", 0)
+                    else:
+                        counts_dict = zone_data.get("original_counts") if isinstance(zone_data.get("original_counts"), dict) else zone_data
+                        current_count = counts_dict.get(
+                            "total",
+                            sum(v for v in counts_dict.values() if isinstance(v, (int, float)))
+                        )
+                human_text_lines.append(f"\t{zone_name}: {int(current_count)}")
+        else:
+            for cat, count in per_category_count.items():
                 human_text_lines.append(f"\t{cat}: {count}")
+        human_text_lines.append(f"TOTAL SINCE {start_timestamp}")
+        # Append zone-wise total counts if available
+        if zone_analysis:
+            human_text_lines.append("\tZones (total):")
+            for zone_name, zone_data in zone_analysis.items():
+                total_count = 0
+                if isinstance(zone_data, dict):
+                    if "total_count" in zone_data:
+                        total_count = zone_data.get("total_count", 0)
+                    else:
+                        counts_dict = zone_data.get("original_counts") if isinstance(zone_data.get("original_counts"), dict) else zone_data
+                        total_count = counts_dict.get(
+                            "total",
+                            sum(v for v in counts_dict.values() if isinstance(v, (int, float)))
+                        )
+                human_text_lines.append(f"\t{zone_name}: {int(total_count)}")
+        else:
+            for cat, count in total_counts_dict.items():
+                if count > 0:
+                    human_text_lines.append(f"\t{cat}: {count}")
         if alerts:
             for alert in alerts:
                 human_text_lines.append(f"Alerts: {alert.get('settings', {})} sent @ {current_timestamp}")
@@ -481,24 +628,29 @@ class VehicleMonitoringUseCase(BaseProcessor):
         tracking_stats.append(tracking_stat)
         return tracking_stats
 
-    def _generate_business_analytics(self, counting_summary: Dict, alerts: Any, config: VehicleMonitoringConfig,
+    def _generate_business_analytics(self, counting_summary: Dict, zone_analysis: Dict, alerts: Any, config: VehicleMonitoringConfig,
                                     stream_info: Optional[Dict[str, Any]] = None, is_empty=False) -> List[Dict]:
         if is_empty:
             return []
 
-    def _generate_summary(self, summary: dict, incidents: List, tracking_stats: List, business_analytics: List, alerts: List) -> List[str]:
-        lines = {}
-        lines["Application Name"] = self.CASE_TYPE
-        lines["Application Version"] = self.CASE_VERSION
+    def _generate_summary(self, summary: dict, zone_analysis: Dict, incidents: List, tracking_stats: List, business_analytics: List, alerts: List) -> List[str]:
+        """
+        Generate a human_text string for the tracking_stat, incident, business analytics and alerts.
+        """
+        lines = []
+        lines.append("Application Name: "+self.CASE_TYPE)
+        lines.append("Application Version: "+self.CASE_VERSION)
         if len(incidents) > 0:
-            lines["Incidents:"] = f"\n\t{incidents[0].get('human_text', 'No incidents detected')}\n"
+            lines.append("Incidents: "+f"\n\t{incidents[0].get('human_text', 'No incidents detected')}")
         if len(tracking_stats) > 0:
-            lines["Tracking Statistics:"] = f"\t{tracking_stats[0].get('human_text', 'No tracking statistics detected')}\n"
+            lines.append("Tracking Statistics: "+f"\t{tracking_stats[0].get('human_text', 'No tracking statistics detected')}")
         if len(business_analytics) > 0:
-            lines["Business Analytics:"] = f"\t{business_analytics[0].get('human_text', 'No business analytics detected')}\n"
+            lines.append("Business Analytics: "+f"\t{business_analytics[0].get('human_text', 'No business analytics detected')}")
+
         if len(incidents) == 0 and len(tracking_stats) == 0 and len(business_analytics) == 0:
-            lines["Summary"] = "No Summary Data"
-        return [lines]
+            lines.append("Summary: "+"No Summary Data")
+
+        return ["\n".join(lines)]
 
     def _get_track_ids_info(self, detections: list) -> Dict[str, Any]:
         frame_track_ids = set()
@@ -548,6 +700,7 @@ class VehicleMonitoringUseCase(BaseProcessor):
         return f"{hours:02d}:{minutes:02d}:{seconds:.1f}"
 
     def _get_current_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False, frame_id: Optional[str]=None) -> str:
+        """Get formatted current timestamp based on stream type."""
         if not stream_info:
             return "00:00:00.00"
         if precision:
@@ -556,15 +709,22 @@ class VehicleMonitoringUseCase(BaseProcessor):
                     start_time = int(frame_id)/stream_info.get("input_settings", {}).get("original_fps", 30)
                 else:
                     start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
-                return self._format_timestamp_for_video(start_time)
+                stream_time_str = self._format_timestamp_for_video(start_time)
+                
+
+                return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
             else:
                 return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
+
         if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
             if frame_id:
                 start_time = int(frame_id)/stream_info.get("input_settings", {}).get("original_fps", 30)
             else:
                 start_time = stream_info.get("input_settings", {}).get("start_frame", 30)/stream_info.get("input_settings", {}).get("original_fps", 30)
-            return self._format_timestamp_for_video(start_time)
+
+            stream_time_str = self._format_timestamp_for_video(start_time)
+            
+            return self._format_timestamp(stream_info.get("input_settings", {}).get("stream_time", "NA"))
         else:
             stream_time_str = stream_info.get("input_settings", {}).get("stream_info", {}).get("stream_time", "")
             if stream_time_str:
@@ -579,16 +739,31 @@ class VehicleMonitoringUseCase(BaseProcessor):
                 return self._format_timestamp_for_stream(time.time())
 
     def _get_start_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False) -> str:
+        """Get formatted start timestamp for 'TOTAL SINCE' based on stream type."""
         if not stream_info:
             return "00:00:00"
+        
         if precision:
-            if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-                return "00:00:00"
+            if self.start_timer is None:
+                self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+                return self._format_timestamp(self.start_timer)
+            elif stream_info.get("input_settings", {}).get("start_frame", "na") == 1:
+                self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+                return self._format_timestamp(self.start_timer)
             else:
-                return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
-        if stream_info.get("input_settings", {}).get("start_frame", "na") != "na":
-            return "00:00:00"
+                return self._format_timestamp(self.start_timer)
+
+        if self.start_timer is None:
+            self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+            return self._format_timestamp(self.start_timer)
+        elif stream_info.get("input_settings", {}).get("start_frame", "na") == 1:
+            self.start_timer = stream_info.get("input_settings", {}).get("stream_time", "NA")
+            return self._format_timestamp(self.start_timer)
+        
         else:
+            if self.start_timer is not None:
+                return self._format_timestamp(self.start_timer)
+
             if self._tracking_start_time is None:
                 stream_time_str = stream_info.get("input_settings", {}).get("stream_info", {}).get("stream_time", "")
                 if stream_time_str:
@@ -600,10 +775,57 @@ class VehicleMonitoringUseCase(BaseProcessor):
                         self._tracking_start_time = time.time()
                 else:
                     self._tracking_start_time = time.time()
+
             dt = datetime.fromtimestamp(self._tracking_start_time, tz=timezone.utc)
             dt = dt.replace(minute=0, second=0, microsecond=0)
             return dt.strftime('%Y:%m:%d %H:%M:%S')
 
+    def _format_timestamp(self, timestamp: Any) -> str:
+        """Format a timestamp so that exactly two digits follow the decimal point (milliseconds).
+
+        The input can be either:
+        1. A numeric Unix timestamp (``float`` / ``int``) – it will first be converted to a
+           string in the format ``YYYY-MM-DD-HH:MM:SS.ffffff UTC``.
+        2. A string already following the same layout.
+
+        The returned value preserves the overall format of the input but truncates or pads
+        the fractional seconds portion to **exactly two digits**.
+
+        Example
+        -------
+        >>> self._format_timestamp("2025-08-19-04:22:47.187574 UTC")
+        '2025-08-19-04:22:47.18 UTC'
+        """
+
+        # Convert numeric timestamps to the expected string representation first
+        if isinstance(timestamp, (int, float)):
+            timestamp = datetime.fromtimestamp(timestamp, timezone.utc).strftime(
+                '%Y-%m-%d-%H:%M:%S.%f UTC'
+            )
+
+        # Ensure we are working with a string from here on
+        if not isinstance(timestamp, str):
+            return str(timestamp)
+
+        # If there is no fractional component, simply return the original string
+        if '.' not in timestamp:
+            return timestamp
+
+        # Split out the main portion (up to the decimal point)
+        main_part, fractional_and_suffix = timestamp.split('.', 1)
+
+        # Separate fractional digits from the suffix (typically ' UTC')
+        if ' ' in fractional_and_suffix:
+            fractional_part, suffix = fractional_and_suffix.split(' ', 1)
+            suffix = ' ' + suffix  # Re-attach the space removed by split
+        else:
+            fractional_part, suffix = fractional_and_suffix, ''
+
+        # Guarantee exactly two digits for the fractional part
+        fractional_part = (fractional_part + '00')[:2]
+
+        return f"{main_part}.{fractional_part}{suffix}"
+
     def _count_categories(self, detections: list, config: VehicleMonitoringConfig) -> dict:
         counts = {}
         for det in detections:
@@ -702,9 +924,6 @@ class VehicleMonitoringUseCase(BaseProcessor):
         }
         return canonical_id
 
-    def _format_timestamp(self, timestamp: float) -> str:
-        return datetime.fromtimestamp(timestamp, timezone.utc).strftime('%Y-%m-%d %H:%M:%S UTC')
-
     def _get_tracking_start_time(self) -> str:
         if self._tracking_start_time is None:
             return "N/A"
